import axios from 'axios';
import { sleep } from '../utils/sleep';
import { Chapter } from '../types';

export const getAllChapters = async (
  id: string,
  userAgent: string,
): Promise<Chapter[]> => {
  let nextPage = false;
  let offset = 0;

  const chapters: Chapter[] = [];
  do {
    const response = await axios.get<{
      data: Chapter[];
      limit: number;
      offset: number;
      total: number;
    }>(`https://api.mangadex.org/manga/${id}/feed`, {
      params: {
        translatedLanguage: ['en'],
        offset,
      },
      headers: {
        'User-Agent': userAgent,
      },
    });

    chapters.push(...response.data.data);

    // Instead of handling timeout & rate limiting, we're just sleeping for a bit
    // Gross - but it works! This isn't time-critical code.
    await sleep(500);

    if (response.data.offset < response.data.total - response.data.limit) {
      offset += response.data.limit;
      nextPage = true;
    } else {
      nextPage = false;
    }
  } while (nextPage);

  return chapters;
};

export const getLatestChapter = (chapters: Chapter[]): Chapter => {
  let latestFoundByChapterNumber = {
    id: '',
    chapter: 0,
  };

  for (const chapter of chapters) {
    const chapterId = chapter.id;
    const chapterNumber = parseFloat(chapter.attributes.chapter);

    if (chapterNumber > latestFoundByChapterNumber.chapter) {
      latestFoundByChapterNumber = {
        id: chapterId,
        chapter: chapterNumber,
      };
    }
  }

  return chapters.find(
    (chapter) => chapter.id === latestFoundByChapterNumber.id,
  );
};
